Prompt = {
"-DOC":"""
<任务>:请为一个指定的方法编写或者重新Doc。
<相关信息>:{context}
<规则>:
    1.只要生成文档，不要返回方法的定义部分,返回的文档内容，请用如下格式返回:
        ```plaintext
        生成的文档内容
        ```。
<Doc的模板定义>:
    [功能说明]:说明一下这个功能的总体功能。
    [设计思想]:这个方法的基本设计思想。
    [实现步骤]:罗列出这个方法实现过程中的主要步骤,如果当前的方法并没有实现.
            这里请你根据当前的doc文档,以及上下文里提到的相关的可以调用方法的定义,对当前这个方法进行设计。
    :param xxxx: 参数介绍  
    :return: 返回值的说明
<目标方法>:\n{method_info}\n
""",
"-CODE":"""
<任务>:请为一个指定的方法编写代码实现。
<相关信息>:{context}
<规则>:请以精简的方式回复，不要给出过多的解释，用例，只要返回该方法的代码，或者该方法相关的xml部分。
<目标方法>:\n{method_info}\n
""",
"-DES":"""
<任务>:请为一个指定的方法编写设计文档。
<相关信息>:{context}
<规则>:只要生成文档，不要返回方法的定义
<设计文档的模板定义>:
    [功能说明]:说明一下这个功能的总体功能。
    [设计思想]:这个方法的基本设计思想。
    [实现步骤]:罗列出这个方法实现过程中的主要步骤,如果当前的方法并没有实现.
            这里请你根据当前的doc文档,以及上下文里提到的相关的可以调用方法的定义,对当前这个方法进行设计。
<目标方法>:\n{method_info}\n
""",
"-MOD":"""
<任务>:请为下面指定的方法，基于提供的<修改信息>进行修改和优化。
<相关信息>:{context}
<规则>:只要生成方法的代码实现，不要提供使用案例。
<修改信息>:{change_info}\n
<目标方法>:\n{method_info}\n
"""
}
